<?php

/*	Vulpes - Gateway Access Manager
	Copyright © 2007, 2008 LinFox Serviços em Informatica LTDA.

	This file is part of Vulpes.

	Vulpes is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Vulpes is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Vulpes.  If not, see <http://www.gnu.org/licenses/>. */

require_once 'lib/controller.php';
require_once 'dao/netcontrol_dao.php';

class netcontrol_ctl extends controller
{
	private $contracts;
	protected $netcontrol_dao;

	public function __construct()
	{
		$this->netcontrol_dao = new netcontrol_dao();
		$this->contracts = $this->netcontrol_dao->fetch_contracts(date('l'),date('H:i:s'));
	}

	function refresh_dhcp()
	{
		$config = config::get_instance();
		$out_iface = $config->iface;
		$commands = array();
		$dhcpd_conf = stripcslashes($config->dhcpd_header);

		foreach ($this->contracts as $contract) {
			if($contract->ip != '' && $contract->mac != '') {
				$dhcpd_conf .= "\n";
				$dhcpd_conf .= "host ".$contract->username.$contract->handle." {\n";
				$dhcpd_conf .= "\thardware ethernet ".strtoupper($contract->mac).";\n";
				$dhcpd_conf .= "\tfixed-address ".$contract->ip.";\n";
				$dhcpd_conf .= "}\n";
			}
		}
		$file = fopen('/etc/dhcp3/dhcpd.conf','w');
		fwrite($file, $dhcpd_conf);
		fclose($file);
		return '<netcontrol><dhcp>'.exec('sudo /etc/init.d/dhcp3-server restart').'</dhcp></netcontrol>';
	}
	
	function refresh_iptables()
	{
		$config = config::get_instance();
		$out_iface = $config->iface;
		$commands = array();
		$iptables = '';
		$iptables_nat = '';
		$iptables_mangle = '';

		foreach ($this->contracts as $contract) {
			if ($contract->ip != ''
			&& $contract->mac != ''
			&& ($contract->status == 'L' || $contract->status == 'P') 
			&& $contract->scheduled
			&& ($contract->active || !$contract->auth)) {
				$iptables_nat .= '-A PREROUTING -s '.$contract->ip.' -p tcp --dport 80 ';
				if ($contract->bind_mac) $iptables_nat .= '-m mac --mac-source '.strtoupper($contract->mac);
				if ($contract->proxy_port) $iptables_nat .= ' -j REDIRECT --to-port '.$contract->proxy_port."\n";
				else $iptables_nat .= " -j ACCEPT\n";

				$iptables_mangle .= '-A PREROUTING -s '.$contract->ip." -j MARK --set-mark 3".$contract->handle."\n";
				$iptables_mangle .= '-A INPUT -s '.$contract->ip." -j MARK --set-mark 3".$contract->handle."\n";
				
				if ($contract->ttl_limit) {
					$iptables_filter .= '-A FORWARD -s '.$contract->ip." -m ttl --ttl-gt 128 -j DROP\n";
					$iptables_filter .= '-A FORWARD -s '.$contract->ip.' -m ttl --ttl-gt '.(127 - $contract->ttl_limit)." -j ACCEPT\n";
					$iptables_filter .= '-A FORWARD -s '.$contract->ip." -m ttl --ttl-gt 64 -j DROP\n";
					$iptables_filter .= '-A FORWARD -s '.$contract->ip.' -m ttl --ttl-gt '.(63 - $contract->ttl_limit)." -j ACCEPT\n";
					$iptables_filter .= '-A FORWARD -s '.$contract->ip.' -m ttl --ttl-lt '.(64 - $contract->ttl_limit)." -j DROP\n";
				}
				
				$iptables_filter .= '-A FORWARD -s '.$contract->ip;
				if ($contract->bind_mac) $iptables_filter .= ' -m mac --mac-source '.strtoupper($contract->mac);
				$iptables_filter .= " -j ACCEPT\n";

				$iptables_filter .= '-A FORWARD -d '.$contract->ip." -j ACCEPT\n";

				$iptables_filter .= '-A INPUT -s '.$contract->ip;
				if ($contract->bind_mac) $iptables_filter .= ' -m mac --mac-source '.strtoupper($contract->mac);
				$iptables_filter .= " -j ACCEPT\n";

				$iptables_filter .= '-A OUTPUT -s '.$contract->ip." -j ACCEPT\n";
			}
		}
		$iptables = stripcslashes($config->iptables);
		$iptables = str_replace('{vulpes_nat}',$iptables_nat,$iptables);
		$iptables = str_replace('{vulpes_mangle}',$iptables_mangle,$iptables);
		$iptables = str_replace('{vulpes_filter}',$iptables_filter,$iptables);
		$file_iptables = fopen('/etc/firewall','w');
		fwrite($file_iptables,$iptables);
		fclose($file_iptables);
		exec('sudo /sbin/iptables-restore < /etc/firewall');
	}

	function refresh_tc()
	{
		$config = config::get_instance();
		$out_iface = $config->iface;
		$used_interfaces = array();

		$commands = "#!/bin/sh\n";
		$commands .= 'sudo /sbin/tc qdisc del dev '.$config->iface." root\n";
		$commands .= 'sudo /sbin/tc qdisc add dev '.$config->iface." root handle 1: htb default 3\n";

		foreach ($this->contracts as $contract) {
			if($contract->ip != '' && $contract->mac != '') {
				if (!$used_interfaces[$contract->iface]) {
					$commands .= 'sudo /sbin/tc qdisc del dev '.$contract->iface." root\n";
					$commands .= 'sudo /sbin/tc qdisc add dev '.$contract->iface." root handle 1: htb default 3\n";
					$used_interfaces[$contract->iface] = true;
				}
				if ($contract->status == 'L' || $contract->status == 'P') {
					$commands .= 'sudo /sbin/tc class add dev '.$out_iface.' parent 1:0 classid 1:1'.$contract->handle.' htb rate '.$contract->upload."kbit ceil 0 burst 15k\n";
					$commands .= 'sudo /sbin/tc qdisc add dev '.$out_iface.' parent 1:1'.$contract->handle.' handle 1'.$contract->handle.": sfq perturb 10\n";
					$commands .= 'sudo /sbin/tc class add dev '.$contract->iface.' parent 1:0 classid 1:2'.$contract->handle.' htb rate '.$contract->download."kbit ceil 0 burst 15k\n";
					$commands .= 'sudo /sbin/tc qdisc add dev '.$contract->iface.' parent 1:2'.$contract->handle.' handle 2'.$contract->handle.": sfq perturb 10\n";
					$commands .= 'sudo /sbin/tc filter add dev '.$out_iface.' parent 1:0 protocol ip prio 1 handle 3'.$contract->handle.' fw flowid 1:1'.$contract->handle."\n";
					$commands .= 'sudo /sbin/tc filter add dev '.$contract->iface.' parent 1:0 protocol ip prio 1 u32 match ip dst '.$contract->ip.' flowid 1:2'.$contract->handle."\n";
				}
			}
		}
		$file = fopen('/etc/tcrc','w');
		fwrite($file, $commands);
		fclose($file);
		exec('/etc/tcrc');
	}
}

?>
