<?php

/*	LibFox - Simple library for XML-based backends.
	Copyright © 2007, 2008 LinFox Serviços em Informatica LTDA.

	This file is part of LibFox.

	LibFox is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	LibFox is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with LibFox.  If not, see <http://www.gnu.org/licenses/>. */

require_once 'lib/dao.php';

/**
 * boleto_dao
 * 
 * @package boleto
 */
class boleto_dao extends dao
{
	public $key = 'invoice_id';
	protected $table = 'invoices';
	protected $fields_rel_all = array(
		'client_username' => array('clients' => 'username'),
		'client_name' => array('clients' => 'name'),
		'client_address' => array('clients' => 'address'),
		'client_address2' => array('clients' => 'address2'),
		'client_district' => array('clients' => 'district'),
		'client_city' => array('clients' => 'district'),
		'client_postal_code' => array('clients' => 'postal_code'),
		'client_state' => array('clients' => 'state'),
		'client_timestamp' => array('clients' => 'timestamp'),
		'plan_name' => array('plans' => 'name'),
		'account_bank_code' => array('accounts' => 'bank_code'),
		'account_agency' => array('accounts' => 'agency'),
		'account_number' => array('accounts' => 'account_number'),
		'account_carteira' => array('accounts' => 'carteira'),
		'account_convention' => array('accounts' => 'convention'),
		'account_currency' => array('accounts' => 'currency'),
		'account_owner_name' => array('accounts' => 'owner_name'),
		'account_owner_cpf_cnpj' => array('accounts' => 'owner_cpf_cnpj'),
		'account_owner_address' => array('accounts' => 'owner_address'),
		'account_owner_city' => array('accounts' => 'owner_city'),
		'account_owner_state' => array('accounts' => 'owner_state'),
		'account_payment_number' => array('accounts' => 'payment_number'),
		'account_payment_info1' => array('accounts' => 'payment_info1'),
		'account_payment_info2' => array('accounts' => 'payment_info2'),
		'account_payment_info3' => array('accounts' => 'payment_info3'),
		'account_payment_info4' => array('accounts' => 'payment_info4'),
		'account_payment_instructions1' => array('accounts' => 'payment_instructions1'),
		'account_payment_instructions2' => array('accounts' => 'payment_instructions2'),
		'account_payment_instructions3' => array('accounts' => 'payment_instructions3'),
		'account_payment_instructions4' => array('accounts' => 'payment_instructions4')
	);
	protected $joins = array(
		'payments.invoice_id' => 'invoices.invoice_id',
		'contracts.handle' => 'invoices.handle',
		'clients.client_id' => 'contracts.client_id',
		'plans.plan_id' => 'contracts.plan_id',
		'accounts.account_id' => 'plans.account_id' );
	public $group_by = 'invoice_id';

	public function __construct()
	{
		parent::__construct();
	}

	public function fetch($id = 1)
	{
		$sql = '
			SELECT '.$this->sql_fetch_fields().',
			SUM(`payments`.`value`) AS `paid`,
			`invoices`.`value` - SUM(`payments`.`value`) AS `net_value`
			FROM `'.$this->table.'`'
			.$this->sql_joins()
			.$this->sql_where($id)
			.$this->sql_group_by()
			.$this->sql_order_by();
		$result = $this->db->query($sql);
		
		if (!$model = $result->fetchObject($this->model)) {
			throw new Exception(lang::not_found.': '.$id.' '.lang::in.' '.$this->table.'.'.$this->key);
			return false;
		}
		$result->closeCursor();
		$model->set_dao($this); 
		return $model;
	}
	
	/**
	 * Fetch an array with models from the database.
	 *
	 * @param mixed $id
	 * @param string $keys example 'table.foo,table.bar'
	 * @return array An array with the models.
	 */
	public function fetch_all($search_model = false)
	{
		$models = array();
		$sql  = 'SELECT '.$this->sql_fetch_all_fields().',
			SUM(`payments`.`value`) AS `paid`,
			`invoices`.`value` - SUM(`payments`.`value`) AS `net_value`
			FROM `'.$this->table.'`'
			.$this->sql_joins()
			.$this->sql_where($search_model)
			.$this->sql_group_by()
			.$this->sql_order_by();

		$result = $this->db->query($sql);
		while ($model = $result->fetchObject($this->model)) {
			$model->set_dao($this);
			$models[] = $model;
		}
		return $models;
	}
	
}

?>