<?php

/*	Vulpes - Gateway Access Manager
	Copyright © 2007, 2008 LinFox Serviços em Informatica LTDA.

	This file is part of Vulpes.

	Vulpes is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Vulpes is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Vulpes.  If not, see <http://www.gnu.org/licenses/>. */

require_once 'lib/dao.php';
require_once 'dao/contract_dao.php';
require_once 'mdl/client.php';

/**
 * Client Data Access Object
 * @author Alwin Garside <alwin@linfox.com.br>
 * @version 0.3
 * @package client
 */
class client_dao extends dao
{
	public $key = 'client_id';
	protected $model = 'client';
	protected $table = 'clients';
	protected $fields_all = 'client_id,username,name,email,active';
	protected $joins = array(
		'contracts.client_id' => 'clients.client_id',
		'plans.plan_id' => 'contracts.plan_id' );
	protected $search_fields = 'clients.username,clients.name,clients.email,contracts.ip,contracts.mac,plans.name';
	protected $delete_rel = 'client_sessions,contracts';
	protected $hash = 'sha512';
	public $order_by = 'name,username';
	public $group_by = 'client_id';
	
	/**
	 * Constructor for client_dao
	 *
	 * @param mysqli $db mysqli connection
	 */
	public function __construct()
	{
		parent::__construct();
	}

	public function activate($client_id)
	{
		$client = $this->fetch($client_id);
		$contract_dao = new contract_dao();
		$contracts = $contract_dao->fetch_all($client);
		$query_activate = 'UPDATE `'.$this->table.'` SET `active` = 1 WHERE `'.$this->key.'` = "'.$client->client_id.'"';
		if ($this->db->query($query_activate)) {
			exec('echo "'.date('Y-m-d H:i:s').': Activating '.$client_id.'" >> /var/log/vulpes.log');
			foreach ($contracts as $contract) {
					$iptables_nat = 'sudo /sbin/iptables -t nat -I PREROUTING -s '.$contract->ip.' -p tcp --dport 80 ';
						if ($contract->bind_mac) $iptables_nat .= '-m mac --mac-source '.strtoupper($contract->mac);
						if ($contract->proxy_port) $iptables_nat .= ' -j REDIRECT --to-port '.$contract->proxy_port."\n";
						else $iptables_nat .= " -j ACCEPT\n";

					$iptables_mangle = 'sudo /sbin/iptables -A PREROUTING -s '.$contract->ip." -j MARK --set-mark 3".$contract->handle."\n";
					$iptables_mangle .= 'sudo /sbin/iptables -A INPUT -s '.$contract->ip." -j MARK --set-mark 3".$contract->handle."\n";	

					$iptables_filter = 'sudo /sbin/iptables -A FORWARD -s '.$contract->ip;
					if ($contract->bind_mac) $iptables_filter .= ' -m mac --mac-source '.strtoupper($contract->mac);
					$iptables_filter .= " -j ACCEPT\n";

					$iptables_filter .= 'sudo /sbin/iptables -A FORWARD -d '.$contract->ip." -j ACCEPT\n";

					$iptables_filter .= 'sudo /sbin/iptables -A INPUT -s '.$contract->ip;
					if ($contract->bind_mac) $iptables_filter .= ' -m mac --mac-source '.strtoupper($contract->mac);
					$iptables_filter .= " -j ACCEPT\n";

					$iptables_filter .= 'sudo /sbin/iptables -A OUTPUT -s '.$contract->ip." -j ACCEPT\n";
					
//					exec('echo "'.date('Y-m-d H:i:s').': '.$iptables_nat.$iptables_mangle.$iptables_filter.'" >> /var/log/vulpes.log');	
					exec($iptables_nat.$iptables_mangle.$iptables_filter);
			}
			return true;
		} else {
			throw new Exception('couldnt_activate_client');
			return false;
		}
	}

	public function disactivate($client_id)
	{
		$query_disactivate = 'UPDATE `'.$this->table.'` SET `active` = 0 WHERE `'.$this->key.'` = "'.$client_id.'"';
		if ($this->db->query($query_disactivate)) {
			return true;
		} else {
			throw new Exception('couldnt_disactivate_client');
		}
	}

	public function update_password(&$client,$password)
	{
		$sql_update_password = '
			UPDATE `clients`
			SET `password` = "'.hash('sha512', $password).'"
			WHERE `client_id` = "'.$client->client_id.'"';
		if (!$this->db->query($sql_update_password)) {
			throw new Exception('password_not_saved');
			return false;
		}
		return true;
	}
	
	public function verify_password(&$client,$password1)
	{
		$sql_password2 = '
			SELECT `password`
			FROM `'.$this->table.'`
			WHERE `'.$this->key.'` = "'.$this->key.'"';
		if (!$result_password2 = $this->db->query($sql_password2)) {
			throw new Exception('password_query_failed');
			return false;
		} elseif (!$result_password2->num_rows) {
			$result_password->closeCursor();
			throw new Exception('client_or_password_incorrect');
			return false;
		} 
		$password2 = $result_password2->fetchObject();
		$result_password->closeCursor();
		if ($password2->password != $password1) {
			return false;
		} else {
			return true;
		}
	}

	/**
	 * Authenticate a client
	 * 
	 * em caso que nao, tenta autenticar usando os valores no $_POST
	 * em caso que o usuario é autenticado, os dados sao guardados nas variaveis desse class
	 */
	public function auth()
	{
		if ($_POST['action'] == 'logout' || $_GET['action'] == 'logout') {
			$_SESSION = array();
			session_destroy();
			return false;
		} elseif (isset($_POST['lf_client_username']) && isset($_POST['lf_client_password'])) {
			$sql_client = '
				SELECT *
				FROM `'.$this->table.'`
				WHERE `username` = "'.get('client_username').'"
					AND `password` = "'.hash($this->hash, $_POST['lf_client_password']).'"';
			$result_client = $this->db->query($sql_client);

			$client = $result_client->fetchObject($this->model);
			$result_client->closeCursor();
			if ($client) {
				$_SESSION['vulpes_client_session'] = true;
				$_SESSION['vulpes_client_username'] = $client->username;
				return $client;
			} else {
				return false;
			}
		} elseif ($_SESSION['vulpes_client_session'] == true) {
			$sql_client = '
				SELECT *
				FROM `'.$this->table.'`
				WHERE `username` = "'.$_SESSION['vulpes_client_username'].'"';
			$result_client = $this->db->query($sql_client);
			$client = $result_client->fetchObject($this->model);
			$result_client->closeCursor();
			if ($client) {
				return $client;
			}
		}
		return false;
	}
	
	public function update_timestamp(&$client_id)
	{
		$sql_timestamp = '
			UPDATE `'.$this->table.'`
			SET `timestamp` = "'.date('YmdHis').'"
			WHERE `client_id` = "'.$client_id.'"';
		if ($this->db->query($sql_timestamp)) return true;
		else return false;
	}
}

?>
