<?php

/*	Vulpes - Gateway Access Manager
	Copyright © 2007, 2008 LinFox Serviços em Informatica LTDA.

	This file is part of Vulpes.

	Vulpes is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Vulpes is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Vulpes.  If not, see <http://www.gnu.org/licenses/>. */

require_once 'lib/dao.php';
require_once 'mdl/contract.php';

class contract_dao extends dao
{
	public $key = 'handle';
	public $foreign_keys = 'client_id,plan_id';
	public $other_keys = 'ip';
	protected $model = 'contract';
	protected $table = 'contracts';
	protected $fields_all = 'handle,client_id,plan_id,ip,mac,date,status,network';
	protected $fields_rel = array(
		'plan_price' => array('plans' => 'price')
	);
	protected $fields_rel_all = array(
		'client_username' => array('clients' => 'username'),
		'client_name' => array('clients' => 'name'),
		'client_active' => array('clients' => 'active'),
		'client_timestamp' => array('clients' => 'timestamp'),
		'plan_name' => array('plans' => 'name'),
		'user_username' => array('users' => 'username')
	);
	protected $fields_sum = array(
		'plan_price' => array('plans' => 'price')
	);
	protected $select_sql = 'DATE_FORMAT(`contracts`.`date`, \'%Y-%m\') AS `month`';
	protected $joins = array(
		'clients.client_id' => 'contracts.client_id',
		'plans.plan_id' => 'contracts.plan_id',
		'users.user_id' => 'contracts.user_id'
	);
	protected $search_fields = 'contracts.network,clients.username,clients.name,plans.name';
	public $order_by = 'clients.name,handle';
	public $group_by = 'handle';

	public function __construct()
	{
		parent::__construct();
	}

	public function fetch_cron($a = false, $b = false)
	{
		$config = config::get_instance();
		$models = array();
		$sql  = 'SELECT '.$this->sql_fetch_all_fields().',
			MAX(`invoices`.`date`) AS `last_invoice_date`,
			MAX(`payments`.`date`) AS `last_payment_date`,
			SUM(`payments`.`value`) AS `total_payments_value`,
			SUM(`unpaid_invoices`.`value`) AS `total_unpaid_value`,
			MAX(`unpaid_invoices`.`date`) AS `last_unpaid_invoice_date`
			FROM `'.$this->table.'`
			INNER JOIN `clients` ON (
				`clients`.`client_id` = `contracts`.`client_id` )
			INNER JOIN `plans` ON (
				`plans`.`plan_id` = `contracts`.`plan_id` )
			LEFT JOIN `invoices` ON (
				`invoices`.`handle` = `contracts`.`handle`
				AND `invoices`.`status` != "C" )
			LEFT JOIN `invoices` `unpaid_invoices` ON (
				`unpaid_invoices`.`handle` = `contracts`.`handle`
				AND `unpaid_invoices`.`date` < "'.date('Ymd', mktime(0, 0, 0, date('m'), date('d') - $config->payment_delay, date('Y'))).'"
				AND ( `unpaid_invoices`.`value` > (
						SELECT SUM(`p`.`value`)
						FROM `payments` `p`
						WHERE `p`.`invoice_id` = `unpaid_invoices`.`invoice_id` )
					OR  (
						SELECT SUM(`p`.`value`)
						FROM `payments` `p`
						WHERE `p`.`invoice_id` = `unpaid_invoices`.`invoice_id` ) IS NULL )
				AND `unpaid_invoices`.`status` != "C" )
			LEFT JOIN `payments` ON (
				`payments`.`invoice_id` = `unpaid_invoices`.`invoice_id` )
			LEFT JOIN `users` ON (
				`users`.`user_id` = `contracts`.`user_id`)'
			.$this->sql_group_by()
			.$this->sql_order_by();

		$result = $this->db->query($sql);
		$result->setFetchMode(PDO::FETCH_CLASS,$this->model);
		return $result->fetchAll();
	}

	public function fetch_overdue()
	{
		$models = array();
		$sql  = '
			SELECT '.$this->sql_fetch_all_fields().',
				MAX(`invoices`.`date`) AS `last_invoice`,
				SUM(`payments`.`value`) AS `paid`,
				`invoices`.`value`-SUM(`payments`.`value`) AS `net_value`
			FROM `'.$this->table.'`'
			.$this->sql_joins().'
			INNER JOIN `invoices` ON (
				`invoices`.`handle` = `contracts`.`handle` )
			INNER JOIN `payments` ON (
				`payments`.`invoice_id` = `invoices`.`invoice_id` )
			WHERE `invoices`.`date` < "'.date('Y-m-d').'"'
			.$this->sql_group_by()
			.$this->sql_order_by().'
			HAVING `paid` < `invoices`.`value` OR `paid` IS NULL';
		$result = $this->db->query($sql);
		$result->setFetchMode(PDO::FETCH_CLASS,$this->model);
		return $result->fetchAll();
	}

	public function liberate(&$contract)
	{
		$sql = '
			UPDATE '.$this->table.'
			SET `status` = "L"
			WHERE `handle` = "'.addslashes($contract->handle).'"';
		if ($this->db->query($sql)) {
			$contract->status = 'L';
			return true;
		} else {
			return false;
		}
	}

	public function block(&$contract)
	{
		$sql = '
			UPDATE '.$this->table.'
			SET `status` = "B"
			WHERE `handle` = "'.addslashes($contract->handle).'"';
		if ($this->db->query($sql)) {
			$contract->status = 'B';
			return true;
		} else {
			return false;
		}
	}
}

?>
