<?php

/*	Vulpes - Gateway Access Manager
	Copyright © 2007, 2008 LinFox Serviços em Informatica LTDA.

	This file is part of Vulpes.

	Vulpes is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Vulpes is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Vulpes.  If not, see <http://www.gnu.org/licenses/>. */

require_once 'lib/dao.php';

class invoice_dao extends dao
{
	public $key = 'invoice_id';
	public $foreign_keys = 'handle';
	protected $table = 'invoices';
	protected $fields_rel_all = array(
		'client_id' => array('clients' => 'client_id'),
		'client_username' => array('clients' => 'username'),
		'client_name' => array('clients' => 'name'),
		'client_timestamp' => array('clients' => 'timestamp'),
		'contract_status' => array('contracts' => 'status'),
		'plan_name' => array('plans' => 'name') );
	protected $joins = array(
		'contracts.handle' => 'invoices.handle',
		'clients.client_id' => 'contracts.client_id',
		'plans.plan_id' => 'contracts.plan_id' );
	protected $search_fields = 'invoices.invoice_id,invoices.date,invoices.value,plans.name,clients.username,clients.name';
	protected $where = '`invoices`.`status` <> "C"';
	public $group_by = 'invoice_id';
	public $order_by = 'date';

	function __construct()
	{
		parent::__construct();
	}

	public function fetch($id = 1)
	{
		$sql = '
			SELECT '.$this->sql_fetch_fields().',
				SUM(`payments`.`value`) AS `paid`,
				`invoices`.`value`-SUM(`payments`.`value`) AS `net_value` 
			FROM `'.$this->table.'`'
			.' LEFT JOIN `payments` ON (
				`payments`.`invoice_id` = `invoices`.`invoice_id`
				AND `payments`.`status` <> "C" )'
			.$this->sql_joins()
			.$this->sql_where($id)
			.$this->sql_group_by();
		$result = $this->db->query($sql);
		
		if (!$model = $result->fetchObject($this->model)) {
			throw new Exception(lang::not_found.': '.$id.' '.lang::in.' '.$this->table.'.'.$this->key);
			return false;
		}
		$result->closeCursor();
		$model->set_dao($this); 
		return $model;
	}

	public function fetch_all($search_model = false)
	{
		$sql  = '
			SELECT '.$this->sql_fetch_all_fields().',
				SUM(`payments`.`value`) AS `paid`,
				`invoices`.`value`-SUM(`payments`.`value`) AS `net_value`
			FROM `'.$this->table.'`'
			.' LEFT JOIN `payments` ON (
				`payments`.`invoice_id` = `invoices`.`invoice_id`
				AND `payments`.`status` <> "C" )'
			.$this->sql_joins()
			.$this->sql_where($search_model)
			.$this->sql_group_by()
			.$this->sql_order_by();
		$result = $this->db->query($sql);
		$result->setFetchMode(PDO::FETCH_CLASS,$this->model);
		return $result->fetchAll();
	}

	public function fetch_paid($search_model = false)
	{
		$sql  = 'SELECT '.$this->sql_fetch_all_fields()
			.', SUM(`payments`.`value`) AS `paid`,
				`invoices`.`value`-SUM(`payments`.`value`) AS `net_value` 
			FROM `'.$this->table.'`'
			.' LEFT JOIN `payments` ON (
				`payments`.`invoice_id` = `invoices`.`invoice_id`
				AND `payments`.`status` <> "C" )'
			.$this->sql_joins('INNER')
			.$this->sql_where($search_model)
			.$this->sql_group_by()
			.' HAVING `paid` >= `invoices`.`value`'
			.$this->sql_order_by();
		$result = $this->db->query($sql);
		$result->setFetchMode(PDO::FETCH_CLASS,$this->model);
		return $result->fetchAll();
	}

	public function fetch_pending($search_model = false)
	{
		$sql  = '
			SELECT '.$this->sql_fetch_all_fields().',
				SUM(`payments`.`value`) AS `paid`,
				`invoices`.`value`-SUM(`payments`.`value`) AS `diff` 
			FROM `'.$this->table.'`'
			.' LEFT JOIN `payments` ON (
				`payments`.`invoice_id` = `invoices`.`invoice_id`
				AND `payments`.`status` <> "C" )'
			.$this->sql_joins('INNER')
			.$this->sql_where($search_model)
			.' AND `invoices`.`status` <> "C"'
			.$this->sql_group_by()
			.' HAVING `paid` < `invoices`.`value` OR `paid` IS NULL'
			.$this->sql_order_by();
		$result = $this->db->query($sql);
		$result->setFetchMode(PDO::FETCH_CLASS,$this->model);
		return $result->fetchAll();
	}

	public function fetch_canceled($search_model = false)
	{
		$sql  = 'SELECT '.$this->sql_fetch_all_fields()
			.', SUM(`payments`.`value`) AS `paid`,
				`invoices`.`value`-SUM(`payments`.`value`) AS `net_value` 
			FROM `'.$this->table.'`'
			.' LEFT JOIN `payments` ON (
				`payments`.`invoice_id` = `invoices`.`invoice_id`
				AND `payments`.`status` <> "C" )'
			.$this->sql_joins('INNER')
			.$this->sql_where($search_model,'`invoices`.`status` = "C"')
			.$this->sql_group_by()
			.$this->sql_order_by();
		$result = $this->db->query($sql);
		$result->setFetchMode(PDO::FETCH_CLASS,$this->model);
		return $result->fetchAll();
	}

	public function fetch_all_like($search_model = false, $search = false)
	{
		$models = array();
		$sql = 'SELECT '.$this->sql_fetch_all_fields().',
			SUM(`payments`.`value`) AS `paid`,
				`invoices`.`value`-SUM(`payments`.`value`) AS `net_value`
			FROM `'.$this->table.'`'
			.' LEFT JOIN `payments` ON (
				`payments`.`invoice_id` = `invoices`.`invoice_id`
				AND `payments`.`status` <> "C" )'
			.$this->sql_joins()
			.$this->sql_where_like($search_model, $search, false)
			.$this->sql_group_by()
			.$this->sql_order_by();

		$result = $this->db->query($sql);
		$result->setFetchMode(PDO::FETCH_CLASS,$this->model);
		return $result->fetchAll();
	}

	public function fetch_paid_like($search_model = false, $search = false)
	{
		$models = array();
		$sql = 'SELECT '.$this->sql_fetch_all_fields().',
			SUM(`payments`.`value`) AS `paid`,
				`invoices`.`value`-SUM(`payments`.`value`) AS `net_value`
			FROM `'.$this->table.'`'
			.' LEFT JOIN `payments` ON (
				`payments`.`invoice_id` = `invoices`.`invoice_id`
				AND `payments`.`status` <> "C" )'
			.$this->sql_joins('INNER')
			.$this->sql_where_like($search_model, $search, false)
			.$this->sql_group_by()
			.' HAVING `paid` >= `invoices`.`value`'
			.$this->sql_order_by();

		$result = $this->db->query($sql);
		$result->setFetchMode(PDO::FETCH_CLASS,$this->model);
		return $result->fetchAll();
	}

	public function fetch_pending_like($search_model = false, $search = false)
	{
		$models = array();
		$sql = 'SELECT '.$this->sql_fetch_all_fields().',
			SUM(`payments`.`value`) AS `paid`,
				`invoices`.`value`-SUM(`payments`.`value`) AS `net_value`
			FROM `'.$this->table.'`'
			.' LEFT JOIN `payments` ON (
				`payments`.`invoice_id` = `invoices`.`invoice_id`
				AND `payments`.`status` <> "C" )'
			.$this->sql_joins('INNER')
			.$this->sql_where_like($search_model, $search, false)
			.$this->sql_group_by()
			.' HAVING `paid` < `invoices`.`value` OR `paid` IS NULL'
			.$this->sql_order_by();

		$result = $this->db->query($sql);
		$result->setFetchMode(PDO::FETCH_CLASS,$this->model);
		return $result->fetchAll();
	}

	public function fetch_canceled_like($search_model = false, $search = false)
	{
		$models = array();
		$sql = 'SELECT '.$this->sql_fetch_all_fields().',
			SUM(`payments`.`value`) AS `paid`,
				`invoices`.`value`-SUM(`payments`.`value`) AS `net_value`
			FROM `'.$this->table.'`'
			.' LEFT JOIN `payments` ON (
				`payments`.`invoice_id` = `invoices`.`invoice_id`
				AND `payments`.`status` <> "C" )'
			.$this->sql_joins('INNER')
			.$this->sql_where_like($search_model, $search, false, '`invoices`.`status` = "C"')
			.$this->sql_group_by()
			.$this->sql_order_by();

		$result = $this->db->query($sql);
		$result->setFetchMode(PDO::FETCH_CLASS,$this->model);
		return $result->fetchAll();
	}

	public function emit($search_model = false, $period_time)
	{
		$sql = '
			INSERT INTO `'.$this->table.'` (
				`invoice_id`,
				`handle`,
				`value`,
				`discount`,
				`date`,
				`period_start`,
				`period_end`,
				`description`,
				`comments`,
				`status` )
			SELECT
				`invoices`.`invoice_id`,
				`contracts`.`handle`,
				`plans`.`price` AS `value`,
				0.00 AS `discount`,
				CONCAT("'.date('Y-m-',mktime(0,0,0,date('m',$period_time)+1,date('d',$period_time),date('Y',$period_time))).'",`contracts`.`payday`) AS `date`,
				"'.date('Y-m-',$period_time).'01" AS `period_start`,
				"'.date('Y-m-',$period_time).date('t',$period_time).'" AS `period_end`,
				CONCAT("'.lang::cobranca.' '.lang::plan.' ",`plans`.`name`),
				"",
				"A"
			FROM `contracts`
			INNER JOIN `plans` ON (
				`plans`.`plan_id` = `contracts`.`plan_id` )
			LEFT JOIN `invoices` ON (
				`invoices`.`handle` = `contracts`.`handle`
				AND `invoices`.`period_start` >= "'.date('Y-m-',$period_time).'01"
				AND `invoices`.`period_end` <= "'.date('Y-m-',$period_time).date('t',$period_time).'"
				AND `invoices`.`status` != "C" )'
			.$this->sql_where($search_model,'`contracts`.`date` <= "'.date('Y-m-d',$period_time).'"
				AND (`contracts`.`status` = "L" OR `contracts`.`status` = "I" OR `contracts`.`status` = "P")
				AND `plans`.`price` > 0' )
			.' ON DUPLICATE KEY UPDATE
				`handle` = VALUES(`handle`)';
		$this->db->query($sql);

		$sql = '
			INSERT INTO `'.$this->table.'` (
				`invoice_id`,
				`handle`,
				`value`,
				`discount`,
				`date`,
				`period_start`,
				`period_end`,
				`description`,
				`comments`,
				`status` )
			SELECT
				`invoices`.`invoice_id`,
				`contracts`.`handle`,
				( (`plans`.`price` / '.date('t',$period_time).') * ('.date('t',$period_time).' - (`contracts`.`date` - "'.date('Ym',$period_time).'00") ) ) AS `value`,
				0.00 AS `discount`,
				CONCAT("'.date('Y-m-',mktime(0,0,0,date('m',$period_time)+1,date('d',$period_time),date('Y',$period_time))).'",`contracts`.`payday`) AS `date`,
				`contracts`.`date` AS `period_start`,
				"'.date('Y-m-t',$period_time).'" AS `period_end`,
				CONCAT("'.lang::cobranca.' '.lang::plan.' ",`plans`.`name`) AS `description`,
				"" AS `comments`,
				"A" AS `status`
			FROM `contracts`
			INNER JOIN `plans` ON (
				`plans`.`plan_id` = `contracts`.`plan_id` )
			LEFT JOIN `invoices` ON (
				`invoices`.`handle` = `contracts`.`handle`
				AND `invoices`.`period_start` = `contracts`.`date`
				AND `invoices`.`period_end` <= "'.date('Y-m-',$period_time).date('t',$period_time).'"
				AND `invoices`.`status` != "C" )'
			.$this->sql_where($search_model,'`contracts`.`date` > "'.date('Y-m-d',$period_time).'"
				AND `contracts`.`date` < "'.date('Y-m-t',$period_time).'"
				AND (`contracts`.`status` = "L" OR `contracts`.`status` = "I" OR `contracts`.`status` = "P")
				AND `plans`.`price` > 0' )
			.' ON DUPLICATE KEY UPDATE
				`handle` = VALUES(`handle`),
				`period_start` = VALUES(`period_start`)';
		$this->db->query($sql);
	}

}

?>