<?php

/*	LibFox™ - Simple library for rapid development of web applications.
	Copyright © 2007, 2008 LinFox Serviços em Informatica LTDA.

	This file is part of LibFox.

	LibFox is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	LibFox is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with LibFox.  If not, see <http://www.gnu.org/licenses/>. */

/**
 * Controller class file.
 * 
 * This file contains the controller class.
 * @author Alwin Garside <alwin@linfox.com.br>
 * @copyright Copyright (c) 2007,2008 LinFox Serviços de Informatica LTDA.
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License
 * @version 0.1
 * @package libfox
 */

/**
 * Include requirements.
 */
require_once 'lib/dao.php';
require_once 'lib/changes.php';
require_once 'xoo/xml_xoo.php';

/**
 * Abstract class for controllers
 * 
 * @package libfox
 * @abstract
 */
abstract class controller
{
	/**
	 * Controller's mode.
	 * 
	 * This doesn't have anything to do with the mode used in requests,
	 * it's only used to decide what the group of models is called in the
	 * output.
	 * @var string
	 */
	protected $mode = 'models';
	/**
	 * Controller's model.
	 * 
	 * This doesn't have anything to do with the model used in the dao, but
	 * rather it's only used to decide what the models are called in the
	 * output.
	 * @var string
	 */
	protected $model = 'model';
	/**
	 * Controller's DAO class.
	 * 
	 * Name of the Data Access Object class that the controller should use
	 * to fetch models and manipulate the database.
	 * This DAO class should be located in a file called class_dao.php in
	 * the 'ctl/' subdirectory.
	 * @var string
	 */
	protected $dao = 'dao';
	/**
	 * xoo to use for this controller.
	 *
	 * @var string
	 */
	protected $xoo = 'xml_xoo';
	/**
	 * Template to use.
	 *
	 * @var string
	 */
	public $tpl = 'xml_tpl';

	public function __construct() {	}

	/**
	 * Fetch a single model from the database and output it.
	 *
	 * @return string
	 */
	public function show()
	{
		$dao = new $this->dao();
		$model = $dao->fetch(get($dao->key));
		$xoo = new $this->xoo();
		return $xoo->item($model, $this->model);
	}

	/**
	 * Show all rows from this mode's table.
	 * 
	 * @return string
	 */
	public function show_all()
	{
		$dao = new $this->dao($this->db);
		if (get('limit')) $dao->limit = get('limit');
		if (get('offset')) $dao->offset = get('offset');
		if (get('group_by')) $dao->group_by = get('group_by');
		if (get('order_by')) $dao->order_by = get('order_by');
		$model = $dao->fetch_model();
		$model->get_request();
		$models = $dao->fetch_all($model);
		$xoo = new $this->xoo();
		return $xoo->items($models, $this->model, $this->mode);
	}

	/**
	 * Searches all rows from this mode's table.
	 * 
	 * Result's can be restricted through the search request var.
	 *
	 * @return string
	 */
	public function search_all()
	{
		$dao = new $this->dao($this->db);
		if (get('limit')) $dao->limit = get('limit');
		if (get('offset')) $dao->offset = get('offset');
		if (get('group_by')) $dao->group_by = get('group_by');
		if (get('order_by')) $dao->order_by = get('order_by');
		$model = $dao->fetch_model();
		$model->get_request();
		$models = $dao->fetch_all_like($model, get('search'));

		$xoo = new $this->xoo();
		return $xoo->items($models, $this->model, $this->mode); 
	}

	/**
	 * Create a new model and get it's ID.
	 * 
	 * Return's the ID of the row that was created.
	 *
	 * @return string
	 */
	public function create()
	{
		$dao = new $this->dao($this->db);
		$model = $dao->create();
		return $xoo->item($model, $this->model) . $this->show_all();
	}

	/**
	 * Insert a new model.
	 *
	 * @return unknown
	 */
	public function insert()
	{
		$dao = new $this->dao();
		$model = $dao->fetch_model();
		$model->get_request();
		$dao->insert($model);
		$xoo = new $this->xoo(); 
		return $xoo->item($model, $this->model);
	}

	/**
	 * Update values of a model.
	 *
	 * Return's the updated model in XML
	 * 
	 * @return string
	 */
	public function update()
	{
		$dao = new $this->dao($this->db); 
		$model = $dao->fetch(get($dao->key));
		$model->get_request();
		$dao->update($model);
		$key = $dao->key;
		return $this->show();
	}

	/**
	 * Delete a model.
	 *
	 * @return unknown
	 */
	public function delete()
	{
		$dao = new $this->dao($this->db);
		$dao->delete(get($dao->key));
		return $this->show_all();
	}

}


?>
