/*	Vulpes - Gateway Access Manager
	Copyright © 2007, 2008 LinFox Serviços em Informatica LTDA.

	This file is part of Vulpes.

	Vulpes is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Vulpes is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Vulpes.  If not, see <http://www.gnu.org/licenses/>. */

CREATE DATABASE IF NOT EXISTS `vulpes`
CHARACTER SET utf8 DEFAULT CHARACTER SET utf8
COLLATE utf8_general_ci DEFAULT COLLATE utf8_general_ci;
USE `vulpes`;

CREATE TABLE IF NOT EXISTS `accounts` (
  `account_id` int(10) unsigned NOT NULL auto_increment,
  `name` varchar(32) NOT NULL,
  `bank_name` varchar(255) NOT NULL,
  `bank_code` smallint(3) unsigned zerofill NOT NULL,
  `aceite` char(1) NOT NULL,
  `agency` smallint(4) unsigned NOT NULL,
  `account_number` int(7) unsigned NOT NULL,
  `carteira` tinyint(2) unsigned NOT NULL,
  `convention` varchar(8) NOT NULL,
  `currency` varchar(4) NOT NULL,
  `owner_name` varchar(255) NOT NULL,
  `owner_cpf_cnpj` varchar(20) NOT NULL,
  `owner_address` varchar(255) NOT NULL,
  `owner_city` varchar(255) NOT NULL,
  `owner_state` char(2) NOT NULL,
  `payment_number` bigint(13) unsigned zerofill NOT NULL,
  `payment_info1` varchar(80) NOT NULL,
  `payment_info2` varchar(80) NOT NULL,
  `payment_info3` varchar(80) NOT NULL,
  `payment_info4` varchar(80) NOT NULL,
  `payment_instructions1` varchar(80) NOT NULL,
  `payment_instructions2` varchar(80) NOT NULL,
  `payment_instructions3` varchar(80) NOT NULL,
  `payment_instructions4` varchar(80) NOT NULL,
  PRIMARY KEY (`account_id`)
) DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `clients` (
  `client_id` int(10) unsigned NOT NULL auto_increment,
  `username` varchar(32) NOT NULL,
  `password` binary(128) NOT NULL,
  `name` varchar(255) NOT NULL,
  `address` varchar(255) NOT NULL,
  `address2` varchar(255) NOT NULL,
  `district` varchar(255) NOT NULL,
  `postal_code` varchar(10) NOT NULL,
  `city` varchar(255) NOT NULL,
  `state` varchar(255) NOT NULL,
  `email` varchar(255) NOT NULL,
  `phone` varchar(21) NOT NULL,
  `mobile` varchar(21) NOT NULL,
  `fiscal_number` varchar(15) NOT NULL,
  `identity` varchar(32) NOT NULL,
  `notes` text NOT NULL,
  `status` char(1) NOT NULL,
  `active` boolean NOT NULL default false,
  `timestamp` datetime NOT NULL,
  PRIMARY KEY (`client_id`),
  UNIQUE (`username`)
) DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `config` (
  `config_id` int(10) unsigned NOT NULL auto_increment,
  `sitename` varchar(32) NOT NULL,
  `domain` varchar(255) NOT NULL,
  `dhcpd_header` longtext NOT NULL,
  `iptables` longtext NOT NULL,
  `ip` varchar(15) NOT NULL,
  `cp_port` smallint(5) unsigned NOT NULL,
  `language` varchar(5) NOT NULL,
  `bandwith` varchar(10) NOT NULL,
  `iface` varchar(10) NOT NULL,
  `proxy_port` smallint(5) unsigned NOT NULL,
  `payment_delay` tinyint(3) unsigned NOT NULL,
  `log_prune` int(10) unsigned NOT NULL,
  `template` varchar(32) NOT NULL,
  `block_overdue` boolean NOT NULL,
  `cp_header` longtext NOT NULL,
  `cp_footer` longtext NOT NULL,
  PRIMARY KEY (`config_id`)
) DEFAULT CHARSET=utf8;

INSERT INTO `config` (`config_id`, `sitename`, `domain`, `dhcpd_header`, `iptables`, `ip`, `cp_port`, `language`, `bandwith`, `iface`, `proxy_port`, `payment_delay`, `template`) VALUES
(1, 'Vulpes', 'localhost', 'shared-network name {\n    option domain-name-servers 10.0.0.1;\n    default-lease-time 600;\n    max-lease-time 7200;\n    option subnet-mask 255.255.255.0;\n\n    subnet 10.0.0.0 netmask 255.255.255.0 {\n        option routers 10.0.0.1;\n    }\n}\n', '# Lista de potas registradas pelo IANA: http://www.iana.org/assignments/port-numbers\n# Lista de portas conhecidas: http://en.wikipedia.org/wiki/List_of_TCP_and_UDP_port_numbers\n#\n# ================================================================ #\n# =============== NETWORK ADDRESS TRANSLATION (NAT) ============== #\n# ================================================================ #\n\n*nat\n:PREROUTING ACCEPT [0:0]\n:POSTROUTING ACCEPT [0:0]\n:OUTPUT ACCEPT [0:0]\n\n{vulpes_nat}\n\n        -A PREROUTING -i <private_iface> -p tcp --dport 80 -j REDIRECT --to-port 82\n        -A POSTROUTING -o <public_iface> -j SNAT --to-source <public_ip>\n\nCOMMIT\n\n\n# ================================================================ #\n# =========================== MANGLE ============================= #\n# ================================================================ #\n*mangle\n:INPUT ACCEPT [0:0]\n:OUTPUT ACCEPT [0:0]\n:FORWARD ACCEPT [0:0]\n:PREROUTING ACCEPT [0:0]\n:POSTROUTING ACCEPT [0:0]\n\n{vulpes_mangle}\n\nCOMMIT\n\n\n# ================================================================ #\n# ============================ FILTER ============================ #\n# ================================================================ #\n*filter\n:INPUT ACCEPT [0:0]\n:OUTPUT ACCEPT [0:0]\n:FORWARD DROP [0:0]\n\n# ================================================ #\n# ======== REGRAS DO FORWARD (ROTEAMENTO) ======== #\n\n        -A FORWARD -m state --state RELATED,ESTABLISHED -j ACCEPT\n        -A FORWARD -d 127.0.0.1 -j ACCEPT\n\n#       ping\n        -A FORWARD -p icmp -j ACCEPT\n\n{vulpes_filter}\n\nCOMMIT\n', '10.0.0.1', 82, 'pt_BR', '10000', 'eth1', 3128, 0, 'xml');


CREATE TABLE IF NOT EXISTS `contracts` (
  `handle` smallint(3) unsigned zerofill NOT NULL auto_increment,
  `client_id` int(10) NOT NULL,
  `plan_id` smallint(3) unsigned NOT NULL,
  `user_id` int(10) NOT NULL,
  `ip` varchar(15) NOT NULL,
  `mac` varchar(17) NOT NULL,
  `date` date NOT NULL,
  `canceled` date NOT NULL,
  `payday` smallint(2) unsigned zerofill NOT NULL,
  `network` varchar(32) NOT NULL,
  `status` char(1) NOT NULL,
  PRIMARY KEY (`handle`),
  UNIQUE (`ip`),
  UNIQUE (`mac`)
) DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `invoices` (
  `invoice_id` int(9) unsigned zerofill NOT NULL auto_increment,
  `handle` smallint(3) unsigned zerofill NOT NULL,
  `value` float(8,2) unsigned NOT NULL,
  `discount` float(8,2) unsigned NOT NULL,
  `date` date NOT NULL,
  `period_start` date NOT NULL,
  `period_end` date NOT NULL,
  `description` varchar(255) NOT NULL,
  `comments` longtext NOT NULL,
  `status` char(1) NOT NULL,
  PRIMARY KEY (`invoice_id`)
) DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `log` (
  `log_id` int(10) unsigned NOT NULL auto_increment,
  `user_id` int(10) unsigned NOT NULL,
  `client_id` int(10) unsigned NOT NULL,
  `mode` varchar(32) NOT NULL,
  `action` varchar(32) NOT NULL,
  `request` varchar(255) NOT NULL,
  `date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
  PRIMARY KEY (`log_id`)
) DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `payments` (
  `payment_id` int(10) unsigned NOT NULL auto_increment,
  `invoice_id` int(9) unsigned zerofill NOT NULL,
  `user_id` int(10) unsigned NOT NULL,
  `value` float(8,2) unsigned NOT NULL,
  `interest` float(8,2) unsigned NOT NULL,
  `fines` float(8,2) unsigned NOT NULL,
  `date` date NOT NULL,
  `status` char(1) NOT NULL,
  PRIMARY KEY (`payment_id`)
) DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `plans` (
  `plan_id` smallint(3) unsigned NOT NULL auto_increment,
  `account_id` int(10) unsigned NOT NULL,
  `name` varchar(16) NOT NULL,
  `description` varchar(255) NOT NULL,
  `price` float(8,2) unsigned NOT NULL,
  `download` int(11) unsigned NOT NULL,
  `upload` int(11) unsigned NOT NULL,
  `iface` varchar(10) NOT NULL,
  `auth` boolean NOT NULL,
  `bind_mac` boolean NOT NULL,
  `proxy_port` smallint(5) unsigned NOT NULL,
  `ttl_limit` tinyint(3) unsigned NOT NULL,
  PRIMARY KEY (`plan_id`)
) DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `schedules` (
  `schedule_id` int(10) unsigned NOT NULL auto_increment,
  `plan_id` smallint(3) unsigned NOT NULL,
  `day` varchar(9) NOT NULL,
  `start` time NOT NULL,
  `end` time NOT NULL,
  `upload` int(10) unsigned NOT NULL,
  `download` int(10) unsigned NOT NULL,
  PRIMARY KEY (`schedule_id`)
) DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `users` (
  `user_id` int(10) unsigned NOT NULL auto_increment,
  `username` varchar(32) NOT NULL,
  `name` varchar(255) NOT NULL,
  `password` binary(128) NOT NULL,
  `admin` boolean NOT NULL default false,
  `email` varchar(255) NOT NULL,
  PRIMARY KEY  (`user_id`)
) DEFAULT CHARSET=utf8;

INSERT INTO `users` (`user_id`, `username`, `name`, `password`, `admin`, `email`) VALUES 
(1, 'admin', 'Administrator', 0x6337616434346362616437363261356461306134353266396538353466646331653065376135326133383031356632336633656162316438306239333164643437323633346466616337316364333465626333356431366162376662386139306338316639373531313364366337353338646336396464386465393037376563, true, 'vulpes@linfox.com.br');


CREATE TABLE IF NOT EXISTS `user_groups` (
  `group_id` int(10) unsigned NOT NULL auto_increment,
  `name` varchar(32) NOT NULL,
  `description` varchar(255) NOT NULL,
  PRIMARY KEY (`group_id`)
) DEFAULT CHARSET=utf8;

INSERT INTO `user_groups` (`group_id`, `name`, `description`) VALUES
(1, 'admin', 'Administrators');


CREATE TABLE IF NOT EXISTS `user_members` (
  `member_id` int(10) unsigned NOT NULL auto_increment,
  `user_id` int(10) unsigned NOT NULL,
  `group_id` int(10) unsigned NOT NULL,
  PRIMARY KEY (`member_id`)
) DEFAULT CHARSET=utf8;

INSERT INTO `user_members` (`user_id`, `group_id`) VALUES (1, 1);


CREATE TABLE IF NOT EXISTS `user_permissions` (
  `permission_id` int(10) unsigned NOT NULL auto_increment,
  `group_id` int(10) unsigned NOT NULL,
  `mode` varchar(32) NOT NULL,
  `action` varchar(255) NOT NULL,
  PRIMARY KEY (`permission_id`)
) DEFAULT CHARSET=utf8;

INSERT INTO `user_permissions` (`group_id`, `mode`, `action`) VALUES (1, '_ALL', '_ALL');
