/*	Vulpes - Gateway Access Manager
	Copyright © 2007, 2008 LinFox Serviços em Informatica LTDA.

	This file is part of Vulpes.

	Vulpes is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Vulpes is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Vulpes.  If not, see <http://www.gnu.org/licenses/>. */

CREATE TABLE IF NOT EXISTS `accounts` (
  `account_id` int(10) unsigned NOT NULL auto_increment,
  `name` varchar(32) NOT NULL,
  `bank_name` varchar(255) NOT NULL,
  `bank_code` smallint(3) unsigned zerofill NOT NULL,
  `aceite` char(1) NOT NULL DEFAULT "N",
  `account_number` int(7) unsigned NOT NULL,
  `agency` smallint(4) unsigned NOT NULL,
  `carteira` tinyint(2) unsigned NOT NULL,
  `convention` varchar(8) NOT NULL,
  `currency` varchar(4) NOT NULL,
  `owner_name` varchar(255) NOT NULL,
  `owner_cpf_cnpj` varchar(20) NOT NULL,
  `owner_address` varchar(255) NOT NULL,
  `owner_city` varchar(255) NOT NULL,
  `owner_state` char(2) NOT NULL,
  `payment_number` bigint(13) unsigned zerofill NOT NULL,
  `payment_info1` varchar(80) NOT NULL,
  `payment_info2` varchar(80) NOT NULL,
  `payment_info3` varchar(80) NOT NULL,
  `payment_info4` varchar(80) NOT NULL,
  `payment_instructions1` varchar(80) NOT NULL,
  `payment_instructions2` varchar(80) NOT NULL,
  `payment_instructions3` varchar(80) NOT NULL,
  `payment_instructions4` varchar(80) NOT NULL,
  PRIMARY KEY (`account_id`)
) DEFAULT CHARSET=utf8;


ALTER TABLE `clients`
CHANGE `last_access` `timestamp` DATETIME NOT NULL;


ALTER TABLE `config`
DROP `dhcp_dirty`,
DROP `tc_dirty`,
DROP `iptables_dirty`,
ADD `payment_delay` TINYINT(3) UNSIGNED NOT NULL,
ADD `log_prune` INT(10) UNSIGNED NOT NULL,
ADD `template` VARCHAR(32) NOT NULL,
ADD `block_overdue` BOOLEAN NOT NULL,
ADD `cp_header` longtext NOT NULL,
ADD `cp_footer` longtext NOT NULL;


ALTER TABLE `contracts`
ADD `user_id` INT(10) UNSIGNED NOT NULL AFTER `plan_id`,
ADD `date` DATE NOT NULL,
ADD `canceled` DATE NOT NULL,
ADD `payday` TINYINT(2) UNSIGNED ZEROFILL NOT NULL,
ADD `network` VARCHAR(32) NOT NULL;


CREATE TABLE IF NOT EXISTS `invoices` (
  `invoice_id` int(9) unsigned zerofill NOT NULL auto_increment,
  `handle` smallint(3) unsigned zerofill NOT NULL,
  `value` float(8,2) unsigned NOT NULL,
  `discount` float(8,2) unsigned NOT NULL,
  `date` date NOT NULL,
  `period_start` date NOT NULL,
  `period_end` date NOT NULL,
  `description` varchar(255) NOT NULL,
  `comments` longtext NOT NULL,
  `status` char(1) NOT NULL,
  PRIMARY KEY (`invoice_id`)
) DEFAULT CHARSET=utf8;


/* INSERT INTO `invoices` (
SELECT
	`contracts`.`handle` AS `handle`,
	`plans`.`price` AS `value`,
	"2008-06-10" AS `date`
FROM `contracts`
INNER JOIN `plans` ON (
	`plans`.`plan_id` = `contracts`.`plan_id` );

DELETE FROM `invoices`
WHERE `value` = 0; */


CREATE TABLE IF NOT EXISTS `log` (
  `log_id` int(10) unsigned NOT NULL auto_increment,
  `user_id` int(10) unsigned NOT NULL,
  `client_id` int(10) unsigned NOT NULL,
  `mode` varchar(32) NOT NULL,
  `action` varchar(32) NOT NULL,
  `request` varchar(255) NOT NULL,
  `date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
  PRIMARY KEY (`log_id`)
) DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `payments` (
  `payment_id` int(10) unsigned NOT NULL auto_increment,
  `invoice_id` int(9) unsigned zerofill NOT NULL,
  `user_id` int(10) unsigned NOT NULL,
  `value` float(8,2) unsigned NOT NULL,
  `interest` float(8,2) unsigned NOT NULL,
  `fines` float(8,2) unsigned NOT NULL,
  `date` date NOT NULL,
  `status` char(1) NOT NULL,
  PRIMARY KEY (`payment_id`)
) DEFAULT CHARSET=utf8;


ALTER TABLE `plans`
ADD `account_id` INT(10) UNSIGNED NOT NULL AFTER `plan_id`;


CREATE TABLE IF NOT EXISTS `user_groups` (
  `group_id` int(10) unsigned NOT NULL auto_increment,
  `name` varchar(32) NOT NULL,
  `description` varchar(255) NOT NULL,
  PRIMARY KEY (`group_id`)
) DEFAULT CHARSET=utf8;

INSERT INTO `user_groups` (`group_id`, `name`, `description`) VALUES
(1, 'admin', 'Administrators');


CREATE TABLE IF NOT EXISTS `user_members` (
  `member_id` int(10) unsigned NOT NULL auto_increment,
  `user_id` int(10) unsigned NOT NULL,
  `group_id` int(10) unsigned NOT NULL,
  PRIMARY KEY (`member_id`)
) DEFAULT CHARSET=utf8;

INSERT INTO `user_members` (`user_id`, `group_id`) VALUES (1, 1);


CREATE TABLE IF NOT EXISTS `user_permissions` (
  `permission_id` int(10) unsigned NOT NULL auto_increment,
  `group_id` int(10) unsigned NOT NULL,
  `mode` varchar(32) NOT NULL,
  `action` varchar(255) NOT NULL,
  PRIMARY KEY (`permission_id`)
) DEFAULT CHARSET=utf8;

INSERT INTO `user_permissions` (`group_id`, `mode`, `action`) VALUES (1, '_ALL', '_ALL');
