<?php

/*	Vulpes - Gateway Access Manager
	Copyright © 2000-2002 Pablo Martins F. Costa, João Prado Maia
	Copyright © 200? Daniel William Schultz, Leandro Maniezo
	Copyright © 2006, 2007 Elizeu Alcantara, et al.
	Copyright © 2008 LinFox Serviços em Informatica LTDA. <alwin@linfox.com.br>

	This file is part of Vulpes.

	Vulpes is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Vulpes is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Vulpes.  If not, see <http://www.gnu.org/licenses/>. */

/**
 * invoice_xoo.php
 * 
 * Contains the invoice_xoo class.
 * @author Alwin Garside <yogarine@gmail.com>
 * @license http://www.gnu.org/licenses/gpl-3.0.html GNU General Public License
 */

require_once 'lib/banks.php';
require_once 'xoo/xhtml_xoo.php';

/**
 * XML Output Object for invoices 
 */
class invoice_xoo extends xml_xoo
{
	/**
	 * Returns a Brazilian standard "boleto de cobrança" in XHTML strict.
	 *
	 * @param invoice $invoice
	 * @return string A string containing the boleto in XHTML
	 */
	public function boleto($invoice)
	{
		/**
		 * TODO: Global objects are ugly
		 * Need to either make banks a static, or a singleton.
		 */
		global $banks;

		$config = config::get_instance();
		if (!$banks[$invoice->account_bank_code]) {
			throw new Exception('Codigo de banco invalido.');
		}
		$bank_name = $banks[$invoice->account_bank_code];
		$bank = new $bank_name();
		$code = $bank->get_code($invoice);

		$codigo_digitavel = $bank->monta_codigo_digitavel($code);
		$net_value = number_format(round($invoice->value - $invoice->paid - $invoice->discount, 2),2);
		$bank_code_dv = $bank->modulo_11($invoice->account_bank_code);
		$payment_number = $bank->get_payment_number($invoice);
		$cobranca_dv = 0;
		$agency_code = $invoice->account_agency.'/'.$invoice->account_number.'/'.$cobranca_dv;
		$date = local_date($invoice->date);
		$process_date = date('d/m/Y');
		$period_start = date('d/m/Y',strtotime($invoice->period_start));
		$period_end = date('d/m/Y',strtotime($invoice->period_end));
		$value = number_format($invoice->value,2);
		$discounts = number_format(round($invoice->paid + $invoice->discount, 2),2);

		$result = <<<EOT
	<div style="page-break-after: always">
		<table cellspacing="0" cellpadding="0" style="width: 180mm; border:none">
			<tr>
				<td style="vertical-align: top">
					<h1 style="text-align: center; margin: 0;">$config->sitename</h1>
					<h2 style="text-align: center; margin: 0;">$invoice->description</h2>
				</td>
			</tr>
		</table>

		<div class="cp" style="width: 180mm; text-align: right; border-width: 1pt 0 0 0; border-color: black; border-style: dashed; padding: 0 3pt">
			<strong>Recibo do Sacado</strong>
		</div>

		<table class="boleto_cabecalho" cellspacing="0" cellpadding="0">
			<tr>
				<th style="width: 150px"> 
					<img src="img/$bank_name.jpg" width="150" height="40" alt="" style="vertical-align: bottom"/>
				</th>
				<td class="bc boleto_cabecalho" style="width: 25mm; text-align: center; vertical-align: bottom">
					$invoice->account_bank_code-$bank_code_dv
				</td>
				<td class="ld boleto_cabecalho" style="text-align: right; vertical-align: bottom">
					$codigo_digitavel
				</td>
			</tr>
		</table>

		<table class="boleto" cellspacing="0" cellpadding="0">
			<tr>
				<th colspan="4" style="width: 45%">Cedente</th>
				<th colspan="2" style="width: 26%">Agência/Código do Cedente</th>
				<th style="width: 4%">Espécie</th>
				<th style="width: 7%">Quantidade</th>
				<th style="width: 18%">Nosso número</th>
			</tr>
			<tr>
				<td colspan="4">$invoice->account_owner_name</td>
				<td colspan="2">$agency_code&nbsp;</td>
				<td>$invoice->account_currency&nbsp;</td>
				<td>&nbsp;</td>
				<td class="right">&nbsp;$payment_number</td>
			</tr>
			<tr>
				<th colspan="2" style="width: 37%">Número do documento</th>
				<th colspan="3" style="width: 20%">CPF/CNPJ</th>
				<th colspan="2" style="width: 18%">Vencimento</th>
				<th colspan="2" style="width: 25%">Valor documento</th>
			</tr>
			<tr>
				<td colspan="2">$invoice->invoice_id&nbsp;</td>
				<td colspan="3">$invoice->account_owner_cpf_cnpj&nbsp;</td>
				<td colspan="2">$date&nbsp;</td>
				<td colspan="2" class="right">&nbsp;$value</td>
			</tr>
			<tr>
				<th style="width: 19%">(-) Desconto / Abatimentos</th>
				<th colspan="2" style="width: 19%">(-) Outras deduções</th>
				<th colspan="2" style="width: 19%">(+) Mora / Multa</th>
				<th colspan="2" style="width: 19%">(+) Outros acréscimos</th>
				<th colspan="2" style="width: 24%">(=) Valor cobrado</th>
			</tr>
			<tr>
				<td>$discounts&nbsp;</td>
				<td colspan="2">&nbsp;</td>
				<td colspan="2">&nbsp;</td>
				<td colspan="2">&nbsp;</td>
				<td colspan="2">&nbsp;</td>
			</tr>
			<tr>
				<th colspan="9">Sacado</th>
			</tr>
			<tr>
				<td colspan="9">$invoice->client_name</td>
			</tr>
		</table>

		<table class="boleto" cellspacing="0" cellpadding="0">
			<tr>
				<th class="no_borders" style="border: none">Demonstrativo</th>
				<th class="no_borders" style="border: none; width: 25mm">Autenticação mecânica</th>
			</tr>
			<tr>
				<td class="no_borders" style="border: none">
					$invoice->description<br/>
					Referente ao periodo de $period_start a $period_end<br/>
					$invoice->account_payment_info1<br/>
					$invoice->account_payment_info2<br/>
					$invoice->account_payment_info3<br/>		
					$invoice->account_payment_info4	
				</td>
				<td class="no_borders" style="border: none">&nbsp;</td>
			</tr>
		</table>

		<div style="margin: 0; padding: 0; text-align: right; border-width: 0 0 1pt 0; border-style: dashed; border-color: black; width: 180mm">Corte na linha pontilhada</div>

		<table class="boleto_cabecalho" cellspacing="0" cellpadding="0">
			<tr>
				<th rowspan="2" style="width: 150px"> 
					<img src="img/$bank_name.jpg" width="150" height="40" alt="" style="vertical-align: bottom"/>
				</th>
				<td class="no_borders" style="border: none" colspan="2">&nbsp;</td>
			</tr>
			<tr>
				<td class="bc boleto_cabecalho" style="width: 25mm; height: 5mm; text-align: center; vertical-align: bottom">
					$invoice->account_bank_code-$bank_code_dv
				</td>
				<td class="ld boleto_cabecalho" style="text-align: right; vertical-align: bottom">
					$codigo_digitavel
				</td>
			</tr>
		</table>

		<table class="boleto" cellspacing="0" cellpadding="0">
			<tr>
				<th colspan="6">Local de pagamento</th>
				<th>Vencimento</th>
			</tr>
			<tr>
				<td colspan="6">Pagável em qualquer Banco até o vencimento</td>
				<td class="right">&nbsp;$date</td>
			</tr>
			<tr>
				<th colspan="6">Cedente</th>
				<th>Agência/Código cedente</th>
			</tr>
			<tr>
				<td colspan="6">$invoice->account_owner_name&nbsp;</td>
				<td class="right">&nbsp;$agency_code</td>
			</tr>
			<tr>
				<th>Data do documento</th>
				<th colspan="2">N<span style="text-decoration: underline">o</span> documento</th>
				<th>Espécie doc.</th>
				<th>Aceite</th>
				<th>Data processamento</th>
				<th>Nosso número</th>
			</tr>
			<tr>
				<td>$process_date&nbsp;</td>
				<td colspan="2">$invoice->invoice_id&nbsp;</td>
				<td>$invoice->account_currency&nbsp;</td>
				<td>N&nbsp;</td>
				<td>$process_date&nbsp;</td>
				<td class="right">&nbsp;$payment_number</td>
			</tr>
		
			<tr>
				<th>Uso do banco</th>
				<th>Carteira</th>
				<th>Espécie</th>
				<th colspan="2">Quantidade</th>
				<th>Valor Documento</th>
				<th>(=) Valor documento</th>
			</tr>
			<tr>
				<td>&nbsp;</td>
				<td>$invoice->account_carteira&nbsp;</td>
				<td>$invoice->account_currency&nbsp;</td>
				<td colspan="2">&nbsp;</td>
				<td>&nbsp;</td>
				<td class="right">&nbsp;$value</td>
			</tr>
		
			<tr>
				<th colspan="6" rowspan="2">Instruções (Texto de responsabilidade do cedente)</th>
				<th>(-) Desconto / Abatimentos</th>
			</tr>
			<tr>
				<td class="right">&nbsp;$discounts</td>
			</tr>
			<tr>
				<td colspan="6" rowspan="8" style="vertical-align: top">
					$invoice->account_payment_instructions1<br/>
					$invoice->account_payment_instructions2<br/>
					$invoice->account_payment_instructions3<br/>
					$invoice->account_payment_instructions4
				</td>
				<th>(-) Outras deduções</th>
			</tr>
			<tr>
				<td class="right">&nbsp;</td>
			</tr>
			<tr>
				<th>(+) Mora / Multa</th>
			</tr>
			<tr>
				<td class="right">&nbsp;</td>
			</tr>
			<tr>
				<th>(+) Outros acréscimos</th>
			</tr>
			<tr>
				<td class="right">&nbsp;</td>
			</tr>
			<tr>
				<th>(=) Valor cobrado</th>
			</tr>
			<tr>
				<td class="right">&nbsp;</td>
			</tr>
		
			<tr>
				<th colspan="6">Sacado</th>
				<td rowspan="5">Cód. baixa</td>
			</tr>
			<tr>
				<td colspan="6" style="border-bottom-width: 0">$invoice->client_name&nbsp;</td>
			</tr>
			<tr>
				<td colspan="6" style="border-bottom-width: 0">$invoice->client_address&nbsp;</td>
			</tr>
			<tr>
				<td colspan="6" style="border-bottom-width: 0">$invoice->client_district&nbsp;</td>
			</tr>
			<tr>
				<td colspan="6">
					$invoice->client_postal_code $invoice->client_city - $invoice->client_state&nbsp;
				</td>
			</tr>
			
		</table>

EOT;

		$result .= '
		<table class="boleto" cellspacing="0" cellpadding="0">
			<tr>
				<th class="no_borders" style="border: none">Sacador/Avalista</th>
				<th class="no_borders" style="border: none">Autenticação mecânica - <strong>Ficha de Compensação</strong></th>
			</tr>
		
			<tr>
				<td class="no_borders" style="border: none">'.$this->barcode($code).'&nbsp;</td>
				<td class="no_borders" style="border: none">&nbsp;</td>
			</tr>
		</table>
		
		<div style="border-width: 0 0 1pt 0; border-color: black; border-style: dashed; text-align: right; width: 180mm">
			Corte na linha pontilhada
		</div>
	</div>';

		return $result;
	}

	/**
	 * Returns a barcode in HTML using images to represent the bars.
	 *
	 * @param int $valor
	 * @return string String with barcode in HTML
	 */
	protected function barcode($valor)
	{
		$fino = 1;
		$largo = 3;
		$altura = 50;
		$result = '';

		$barcodes[0] = "00110" ;
		$barcodes[1] = "10001" ;
		$barcodes[2] = "01001" ;
		$barcodes[3] = "11000" ;
		$barcodes[4] = "00101" ;
		$barcodes[5] = "10100" ;
		$barcodes[6] = "01100" ;
		$barcodes[7] = "00011" ;
		$barcodes[8] = "10010" ;
		$barcodes[9] = "01010" ;
		
		for($f1=9; $f1>=0; $f1--){
			for($f2=9; $f2>=0; $f2--){  
				$f = ($f1 * 10) + $f2;
				$texto = "";
				for($i=1;$i<6;$i++){ 
					$texto .=  substr($barcodes[$f1],($i-1),1) . substr($barcodes[$f2],($i-1),1);
				}
				$barcodes[$f] = $texto;
			}
		}
	
		// Desenho da barra
	
		//Guarda inicial
		$result .= '<img src="img/p.png" width="'.$fino.'" height="'.$altura.'" alt=""/>';
		$result .= '<img src="img/b.png" width="'.$fino.'" height="'.$altura.'" alt=""/>';
		$result .= '<img src="img/p.png" width="'.$fino.'" height="'.$altura.'" alt=""/>';
		$result .= '<img src="img/b.png" width="'.$fino.'" height="'.$altura.'" alt=""/>'; 
	
		$texto = $valor;
		if((strlen($texto) % 2) != 0) {
			$texto = "0".$texto;
		}
	
		// Draw dos dados
		while (strlen($texto) > 0) {
			$i = round(esquerda($texto,2));
			$texto = direita($texto,strlen($texto)-2);
			$f = $barcodes[$i];
			for($i=1;$i<11;$i+=2){
				if (substr($f,($i-1),1) == "0") {
					$f1 = $fino;
				} else {
					$f1 = $largo;
				}
				$result .= '<img src="img/p.png" width="'.$f1.'" height="'.$altura.'" alt=""/>'; 
				if (substr($f,$i,1) == "0") {
					$f2 = $fino;
				} else {
					$f2 = $largo;
				}
				$result .= '<img src="img/b.png" width="'.$f2.'" height="'.$altura.'" alt=""/>'; 
			}
		}
	
		// Draw guarda final
		$result .= '<img src="img/p.png" width="'.$largo.'" height="'.$altura.'" alt=""/>';
		$result .= '<img src="img/b.png" width="'.$fino.'" height="'.$altura.'" alt=""/>';
		$result .= '<img src="img/p.png" width="1" height="'.$altura.'" alt=""/>';

		return $result;
	}
	
}

?>